/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

#ifndef LOADER_H_INCLUDED
#define LOADER_H_INCLUDED

/* This is the header for the virtual Loader class, which encapsulates the
   interface that is required for a movie file loader.  There are any
   number of possible instantiations of this interface, all deriving from
   this parent.  The Loader class is used as an argument and interface from
   within the Movie class.  The Loader class has pure-virtual functions,
   and as such may not be instantiated directly.
*/

class Loader{
public:
  // Returns a buffer containing the current frame.
  virtual unsigned char *getCurFrame(void) = 0;

  // Steps one frame forward and returns a buffer of the new current frame.
  // Returns NULL if there are no more frames.
  virtual unsigned char *getNextFrame(void) = 0;

  // Steps one frame back and returns a buffer of the new current frame.
  // Returns NULL if there are no previous frames.
  virtual unsigned char *getPrevFrame(void) = 0;

  // Seeks to a particular frame number, given by 'frameNumber'.  Returns
  // an error if the frameNumber is out of range.
  virtual int seekTo(int frameNumber) = 0;

  // Returns the number of frames in the movie
  virtual int length(void) = 0;

  // Returns the width of a frame in the movie, in pixels
  virtual int width(void) = 0;

  // Returns the height of a frame in the movie, in pixels
  virtual int height(void) = 0;

  // Returns the color-depth of a frame in the movie, in bytes
  virtual int depth(void) = 0;

  // Returns the current frame number.
  virtual int frame(void) = 0;
};

#endif
