/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

#ifndef LOADER_FLATFILEGZFLIP_H_INCLUDED
#define LOADER_FLATFILEGZFLIP_H_INCLUDED

/* This class (Loader_FlatFileGZFlip) implements a Loader for the original
   MovieFile_C type flat-file movie file with gzip compression, flipping
   the frame to support the older encoded movies (which should, eventually,
   all be converted to store the data in proper, TL->BR order).  The
   constructor takes a single argument - the filename.
*/

#include "Loader.h"
#include <stdlib.h>
#include <stdio.h>
#include <zlib.h>

class Loader_FlatFileGZFlip : public Loader{
private:
  FILE *movieFile;
  int w,h,numFrames,frameSize,curFrame;
  unsigned char *frameBuff;
  unsigned char *frameTmp;

public:
  Loader_FlatFileGZFlip(char *filename);

  ~Loader_FlatFileGZFlip(){
    if(frameBuff) delete[] frameBuff;
    gzclose(movieFile);
  };

  // Returns a buffer containing the current frame.
  virtual unsigned char *getCurFrame(void){
    if(curFrame>0){
      gzseek(movieFile,-frameSize,SEEK_CUR);
    }
    gzread(movieFile,frameTmp,frameSize);
    for(int j=h;j>0;j--){
      for(int i=0;i<(w*3);i++){
	frameBuff[((h-j)*(w*3))+i]=frameTmp[((j-1)*(w*3))+i];
      };
    };
    return frameBuff;
  };

  // Steps one frame forward and returns a buffer of the new current frame.
  // Returns NULL if there are no more frames.
  virtual unsigned char *getNextFrame(void){
    if(gzread(movieFile,frameTmp,frameSize)){
      for(int j=h;j>0;j--){
	for(int i=0;i<(w*3);i++){
	  frameBuff[((h-j)*(w*3))+i]=frameTmp[((j-1)*(w*3))+i];
	};
      };
      curFrame++;
      return frameBuff;
    }
    else return NULL;
  };

  // Steps one frame back and returns a buffer of the new current frame.
  // Returns NULL if there are no previous frames.
  virtual unsigned char *getPrevFrame(void){
    if(curFrame>1){
      curFrame--;
      gzseek(movieFile,-(2*frameSize),SEEK_CUR);
      gzread(movieFile,frameTmp,frameSize);
      for(int j=h;j>0;j--){
	for(int i=0;i<(w*3);i++){
	  frameBuff[((h-j)*(w*3))+i]=frameTmp[((j-1)*(w*3))+i];
	};
      };
     return frameBuff;
    }
    else return NULL;
  };

  // Seeks to a particular frame number, given by 'frameNumber'.  Returns
  // an error if the frameNumber is out of range.
  virtual int seekTo(int frameNumber){
    if(frameNumber>=numFrames) return -1;
    curFrame=frameNumber;
    gzseek(movieFile,(sizeof(int)*3)+(frameNumber*frameSize),SEEK_SET);
    return 0;
  };

  // Returns the number of frames in the movie
  virtual int length(void){return numFrames;};

  // Returns the width of a frame in the movie, in pixels
  virtual int width(void){return w;};

  // Returns the height of a frame in the movie, in pixels
  virtual int height(void){return h;};

  // Returns the color-depth of a frame in the movie, in bytes
  virtual int depth(void){return 3;};

  // Returns the current frame number.
  virtual int frame(void){return curFrame;};
};

#endif
