/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

/* This is a "test" movie player - uses the Loader_FlatFile loader,
   the Renderer_GL renderer, and the Movie class abstraction to see
   if everything works.
*/

#include "Movie/Movie.h"
#include "Loader/Loader_FlatFile.h"
#include "Loader/Loader_FlatFileFlip.h"
#include "Loader/Loader_FlatFileGZ.h"
#include "Loader/Loader_FlatFileGZFlip.h"
#include "Renderer/Renderer_GL.h"
#include "td/td.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Movie *movie;

void draw(void){
  if(movie->update()==-1){
    movie->reset();
    movie->update();
  };
  tdSwapBuffers();
};

int main(int argc,char *argv[]){
  FILE *configFile;
  char inBuff[128];

  tdInit(&argc,&argv);

  int myid=tdGetNodeNumber();

  fprintf(stderr,"(%d) - Init\n",myid);

  if(argc<2){
    fprintf(stderr,"USAGE: %s <movieFileBase> [<movieFilePost> [FLIP|NOFLIP]]\n\n",argv[0]);
    exit(-1);
  };

  if(argc<3){
    sprintf(inBuff,"%s.%02d.movie",argv[1],myid);
    movie=new Movie();
    movie->loaderOpen(new Loader_FlatFileFlip(inBuff));
  }
  else{
    movie=new Movie();
    sprintf(inBuff,"%s.%02d.%s",argv[1],myid,argv[2]);
    if(argc<4 || !strcmp(argv[3],"FLIP")){
      if(strrchr(argv[2],'.') && !strcmp(strrchr(argv[2],'.'),".gz")){
	movie->loaderOpen(new Loader_FlatFileGZFlip(inBuff));
      }
      else{
	movie->loaderOpen(new Loader_FlatFileFlip(inBuff));
      };
    }
    else{
      if((strrchr(argv[2],'.') && !strcmp(strrchr(argv[2],'.'),".gz"))){
	movie->loaderOpen(new Loader_FlatFileGZ(inBuff));
      }
      else{
	movie->loaderOpen(new Loader_FlatFile(inBuff));
      };
    };
  };

  fprintf(stderr,"(%d) - Movie created / Loader Opened\n",myid);

  glutInit(&argc,argv);

  movie->rendererOpen(new Renderer_GL());

  fprintf(stderr,"(%d) - Renderer opened\n",myid);

  movie->play();

  glutIdleFunc(draw);
  glutDisplayFunc(draw);

  glutMainLoop();
  
  return 0;

};
