/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

/* This is a "test" movie player - uses all the Loaders, and
   the Renderer_frameCount renderer to test framerates for
   them.
*/

#include "Movie/Movie.h"
#include "Loader/Loader_FlatFile.h"
#include "Loader/Loader_FlatFileFlip.h"
#include "Loader/Loader_FlatFileGZ.h"
#include "Loader/Loader_FlatFileGZFlip.h"
#include "Renderer/Renderer_frameCount.h"
#include "td/td.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

Movie *movie;

int draw(void){
  if(movie->update()==-1){
    return 0;
    movie->reset();
    movie->update();
  };
  tdSwapBuffers();
  return 1;
};

int main(int argc,char *argv[]){
  char inBuff[128];

  tdInit(&argc,&argv);

  int myid=tdGetNodeNumber();

  fprintf(stderr,"(%d) - Init\n",myid);

  if(argc<4){
    fprintf(stderr,"USAGE: %s <LOCAL|GZ> <FLIP|NOFLIP> <moviePrefix>\n",argv[0]);
    exit(-1);
  };

  movie=new Movie();
  sprintf(inBuff,"%s.%02d.movie",argv[3],myid);
  if(!strcmp(argv[1],"GZ")){
    strcat(inBuff,".gz");
    if(!strcmp(argv[2],"FLIP")){
      fprintf(stderr,"(%d) - Using Loader_FlatFileGZFlip\n",myid);
      movie->loaderOpen(new Loader_FlatFileGZFlip(inBuff));
    }
    else{
      fprintf(stderr,"(%d) - Using Loader_FlatFileGZ\n",myid);
      movie->loaderOpen(new Loader_FlatFileGZ(inBuff));
    };
  }
  else if(!strcmp(argv[2],"FLIP")){
    fprintf(stderr,"(%d) - Using Loader_FlatFileFlip\n",myid);
    movie->loaderOpen(new Loader_FlatFileFlip(inBuff));
  }
  else{
    fprintf(stderr,"(%d) - Using Loader_FlatFile\n",myid);
    movie->loaderOpen(new Loader_FlatFile(inBuff));
  };

  fprintf(stderr,"(%d) - Movie created / Loader Opened\n",myid);

  movie->rendererOpen(new Renderer_frameCount());

  fprintf(stderr,"(%d) - Renderer opened\n",myid);

  movie->play();

  while(draw());

  tdFinalize();

  return 0;

};
