/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

#ifndef RENDERER_GL_H_INCLUDED
#define RENDERER_GL_H_INCLUDED

/* This class (Renderer_GL) implements a Renderer for a frame
   of 24-bit RGB data, using glut/glDrawPixels(...).
*/

#include "Renderer.h"
#include <GL/glut.h>
#include <stdlib.h>

class Renderer_GL : public Renderer{
private:
  unsigned char *frameBuff;
  int w;
  int h;

public:
  Renderer_GL(void):
    frameBuff(NULL)
  {};

  ~Renderer_GL(void){if(frameBuff) delete[] frameBuff;};

  // Actually opens the renderer.  Until this point, the width/height/depth
  // of the buffer is (most likely) not known.  The Movie class (or
  // derivative) will call this function with the output of the appropriate
  // queries to the Loader as input.
  virtual int open(int width,int height, int depth);

  // Close the renderer, possibly to be reopened.  This happens when the
  // Movie (or derivative) switches Loaders - the Renderer shouldn't close,
  // but the buffers and context that rely on the width/height/depth of the
  // input buffer should be torn down, 'cause the new Loader may have
  // different values for those parameters.
  virtual int close(void){
    if(frameBuff){
      delete[] frameBuff;
      frameBuff=NULL;
    };
    return 0;
  };

  // Actually render a buffer.  The buffer should consist of
  // width * height * depth pixels (values from open(...) above), exactly,
  // from upper-left to lower-right.  The buffer should not be written to
  // during the operation, nor will the buffer be modified by the draw(...)
  // function.
  virtual int draw(unsigned char *buffer);
};

#endif
