/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mpi.h"

#define DTOR(r) ((r)*0.01745329f)
#define NUMBER_OF_TILES 6

typedef struct _tdconfiguration
{
    int numberOfXTiles;
    int numberOfYTiles;
    int resolutionOfProjectorX;
    int resolutionOfProjectorY;
    float overlap[15][2]; /* Fix this */
}TDConfiguration;

void tdLoadConfiguration(TDConfiguration *tdconfig);

// Set up a render group to swap buffers on

void
tdInit(int *argc, char **argv[])
{
  char processorName[MPI_MAX_PROCESSOR_NAME];
  char display[MPI_MAX_PROCESSOR_NAME+16];
  int pnLength;
  int numprocs,myid;
  
  MPI_Init(argc, argv);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD, &myid);
  //  MPI_Get_processor_name(processorName, &pnLength);
  
  //  sprintf(display, "%s:0", processorName);
  //  setenv("DISPLAY", display, 1);
#ifdef tdDEBUG
  fprintf(stderr,"\n %s is intialized as MPI %d \n",display,myid);
#endif
}/* End of tdInit(int, char **) */
    
void
tdSwapBuffers()
{
  MPI_Barrier(MPI_COMM_WORLD);
 
}/* End of tdSwapBuffers() */

void
tdPerspective(float angle, float aspect, float near, float far)
{
  static float l[NUMBER_OF_TILES],
      r[NUMBER_OF_TILES], 
      b[NUMBER_OF_TILES], 
      t[NUMBER_OF_TILES], 
      n[NUMBER_OF_TILES], 
      f[NUMBER_OF_TILES]; /* Left, Right, Bottom, Top, Near, Far */
  
  static int once = 1;
  
  float width, height, xStep, yStep;
  float d1,d2,left,right,top,bottom;

  int numberOfTiles;
  int resolutionOfProjectorX, resolutionOfProjectorY;
  
  int myid,numprocs;

  int i;
    
  MPI_Comm_rank(MPI_COMM_WORLD, &myid);
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

  if(myid == 0)
    fprintf(stderr,"angle %f aspect %f n %f f %f",angle,
	    aspect,near,far);
  
  if(once)
  {
      TDConfiguration amc;
      float fv;

      tdLoadConfiguration(&amc);

      numberOfTiles = amc.numberOfXTiles * amc.numberOfYTiles;
      resolutionOfProjectorX = amc.resolutionOfProjectorX;
      resolutionOfProjectorY = amc.resolutionOfProjectorY;
      
      if(myid == 0)
	fprintf(stderr,"%d - %d %d %d %d\n",myid,amc.numberOfXTiles,
		amc.numberOfYTiles,amc.resolutionOfProjectorX,
		amc.resolutionOfProjectorY);
      
      if(numprocs != numberOfTiles)
	fprintf(stderr,"Warning: Assuming Full ActiveMural Resolution\n");
      
      for(i=0;i<numberOfTiles;i++)
      {
	  n[i] = near;
	  f[i] = far;
      }/* End of FOR */

      top = near * tan(angle * M_PI / 360.0 );
      bottom = -top;

      left = bottom * aspect;
      right = top * aspect;

      width = 2.0*right;
      height = 2.0*top;

      if(myid == 0)
      {
	  fprintf(stderr,"Left %f Right %f\n",left,right);
	  fprintf(stderr,"Top %f Bottom %f\n",top,bottom);
	  fprintf(stderr,"Width %f Height %f\n",width,height);
      }
      

/* Fix Division */
      xStep = (right-left)/((float)amc.numberOfXTiles);
      yStep = (top-bottom)/((float)amc.numberOfYTiles);

      if(myid == 0)
	  fprintf(stderr,"\nXstep %f Ystep %f\n",xStep,yStep);
            
      /* Top Row of AM */
      l[0] = left + ((amc.overlap[0][0]/((float)resolutionOfProjectorX))*
		     xStep);
      r[0] = left + xStep;
      t[0] = top - (amc.overlap[0][1]/((float)resolutionOfProjectorY));
      b[0] = top - yStep;
	
      if(myid == 0)
	  printf("0 %f %f %f %f\n",l[0],r[0],b[0],t[0]);

      l[1] = r[0] - (xStep - ((amc.overlap[1][0]/
			       ((float)resolutionOfProjectorX))*xStep));
      r[1] = l[1] + xStep;
      t[1] = top - (amc.overlap[1][1]/((float)resolutionOfProjectorY));
      b[1] = top - yStep;

      if(myid == 0)
	  printf("1 %f %f %f %f\n",l[1],r[1],b[1],t[1]);

      l[2] = r[1] - (xStep - (((amc.overlap[2][0]-amc.overlap[1][0])/
			       ((float)resolutionOfProjectorX))*xStep));
      r[2] = l[2] + xStep;
      t[2] = top - (amc.overlap[2][1]/((float)resolutionOfProjectorY));
      b[2] = top - yStep;

      if(myid == 0)
	  printf("2 %f %f %f %f\n",l[2],r[2],b[2],t[2]);

      /* Middle Row of AM */
      
      l[3] = left + ((amc.overlap[3][0]/((float)resolutionOfProjectorX))*
		     xStep);
      r[3] = left + xStep;
      t[3] = b[0] + (yStep - (((amc.overlap[3][1]-amc.overlap[0][1])/((float)resolutionOfProjectorY)) * yStep));
      b[3] = t[3] - yStep;
      
      l[4] = r[3] - (xStep - ((amc.overlap[4][0]/
			       ((float)resolutionOfProjectorX))*xStep));
      r[4] = l[4] + xStep;
      t[4] = b[1] + (yStep - (((amc.overlap[4][1]-amc.overlap[1][1])/((float)resolutionOfProjectorY)) * yStep));
      b[4] = t[4] - yStep;
      
      l[5] = r[4] - (xStep - (((amc.overlap[5][0]-amc.overlap[4][0])/
			       ((float)resolutionOfProjectorX))*xStep));
      r[5] = l[5] + xStep;
      t[5] = b[2] + (yStep - (((amc.overlap[5][1]-amc.overlap[2][1])/((float)resolutionOfProjectorY)) * yStep));
      b[5] = t[5] - yStep;
      
      once = 0;

  }/* End of IF */
  
  //  glFrustum(l[myid],r[myid],b[myid],t[myid],n[myid],f[myid]);
  
}/* End of amPerspective(float, float, float, float) */


void
tdLoadConfiguration(TDConfiguration *amconfig)
{
    FILE *ptr;
    int nXTiles,nYTiles,xRes,yRes;
    float x,y;
    int i;
    
    ptr = fopen("tdconfig","r");
    if(ptr == NULL)
    {
	fprintf(stderr,"Failed to open tdconfig File");
	exit(0);
    }/* End of IF */

    fscanf(ptr,"%d %d %d %d",&nXTiles,&nYTiles,&xRes,&yRes);

    amconfig->numberOfXTiles = nXTiles;
    amconfig->numberOfYTiles = nYTiles;
    amconfig->resolutionOfProjectorX = xRes;
    amconfig->resolutionOfProjectorY = yRes;

    for(i=0;i<(nXTiles*nYTiles);i++)
    {
	fscanf(ptr,"%f %f",&x,&y);
	amconfig->overlap[i][0] = x;
	amconfig->overlap[i][1] = y;
    }/* End of FOR */
    
}/* End of amLoadConfiguration(AMConfiguration *) */

int
tdFinalize()
{
    return MPI_Finalize();
}    

int
tdGetNodeNumber()
{

  int myid;

  MPI_Comm_rank(MPI_COMM_WORLD, &myid);

  return myid;
}
