/* * Copyright (c) 2000 University of Chicago. All Rights Reserved.  *
 * This package is part of the Tiled Display Toolkit.                *
 * The Tiled Display Toolkit is free software. You can redistribute  *
 * it and/or modify it under the terms of the Tiled Display Toolkit  *
 * Public License. You should have received a copy of the            *
 * Tiled Display Toolkit Public License along with this program; if  *
 * not write to the Futures Laboratory at Argonne National           *
 * Laboratory at fl-info@mcs.anl.gov.                                */

/* This is a simple application that creates and SDL movie file, in
   a specific (hard-coded) format.  It can be easily modified to support
   any format required.  NOTE: for performance, the format encoded here
   should exactly match the display format used in the movie player.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

int main(int argc,char *argv[]){
  if(argc<6){
    fprintf(stderr,"USAGE: %s <File Root> <File Extra> <# Frames> <Initial #> <Output file>\n",argv[0]);
    exit(-1);
  };

  char filebase[128];
  char filePost[64];
  char filename[256];
  int numFrames=atoi(argv[3]);
  int initialFrame=atoi(argv[4]);
  int curFrame=initialFrame;

  strcpy(filebase,argv[1]);
  strcpy(filePost,argv[2]);

  FILE *outFile;

  SDL_Surface *mainSurface;
  SDL_Surface *tmp;
  SDL_Surface *tmp2;

  SDL_Init(SDL_INIT_VIDEO);

  mainSurface=SDL_SetVideoMode(1024,768,16,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
  if(!mainSurface){
    fprintf(stderr,"ERROR: Could not open video!\n");
    exit(-1);
  };

  if((outFile=fopen(argv[5],"w"))==NULL){
    fprintf(stderr,"ERROR: Could not open output file!\n");
    SDL_Quit();
    exit(-1);
  };

  sprintf(filename,"%s%04d%s",filebase,curFrame,filePost);
  tmp2=IMG_Load(filename);
  if(!tmp2){
    fprintf(stderr,"ERROR: Could not load initial frame %s!\n",filename);
    fclose(outFile);
    SDL_Quit();
    exit(-1);
  };

  tmp=SDL_DisplayFormat(tmp2);

  fwrite(&numFrames,sizeof(int),1,outFile);
  fwrite(&(tmp->w),sizeof(int),1,outFile);
  fwrite(&(tmp->h),sizeof(int),1,outFile);
  fwrite(&(tmp->format->BitsPerPixel),sizeof(Uint8),1,outFile);
  fwrite(&(tmp->pitch),sizeof(Uint16),1,outFile);
  fwrite(&(tmp->format->Rmask),sizeof(Uint32),1,outFile);
  fwrite(&(tmp->format->Gmask),sizeof(Uint32),1,outFile);
  fwrite(&(tmp->format->Bmask),sizeof(Uint32),1,outFile);
  fwrite(&(tmp->format->Amask),sizeof(Uint32),1,outFile);

  int frameSize=tmp->w*tmp->h*tmp->format->BytesPerPixel;

  SDL_BlitSurface(tmp,NULL,mainSurface,NULL);
  SDL_Flip(mainSurface);

  fwrite(tmp->pixels,frameSize,1,outFile);
  SDL_FreeSurface(tmp);
  SDL_FreeSurface(tmp2);

  curFrame++;

  for(;curFrame<(numFrames+initialFrame);curFrame++){
    sprintf(filename,"%s%04d%s",filebase,curFrame,filePost);
    tmp2=IMG_Load(filename);
    if(!tmp2){
      fprintf(stderr,"FATAL ERROR: Failed to read file %s!\n",filename);
      fclose(outFile);
      SDL_Quit();
      exit(-3);
    };

    tmp=SDL_DisplayFormat(tmp2);
    SDL_BlitSurface(tmp,NULL,mainSurface,NULL);
    SDL_Flip(mainSurface);

    fwrite(tmp->pixels,frameSize,1,outFile);
    SDL_FreeSurface(tmp);
    SDL_FreeSurface(tmp2);
  };

  fclose(outFile);
  fprintf(stdout,"Done!\n");

  SDL_Quit();

  return 0;
};
